@@ -78,13 +78,21 @@ instance Typeable HUnitFailure where
 #ifdef BASE4
 instance Exception HUnitFailure
 
-assertFailure msg = E.throw (HUnitFailure msg)
+assertFailure msg = E.throwIO (HUnitFailure msg)
 
 performTestCase action = 
     do action
        return Nothing
      `E.catches`
       [E.Handler (\(HUnitFailure msg) -> return $ Just (True, msg)),
+
+       -- Re-throw AsyncException, otherwise execution will not terminate on
+       -- SIGINT (ctrl-c).  Currently, all AsyncExceptions are being thrown
+       -- because it's thought that none of them will be encountered during
+       -- normal HUnit operation.  If you encounter an example where this
+       -- is not the case, please email the maintainer.
+       E.Handler (\e -> throw (e :: E.AsyncException)),
+
        E.Handler (\e -> return $ Just (False, show (e :: E.SomeException)))]
 #else
 assertFailure msg = E.throwDyn (HUnitFailure msg)
